function [s] = run_random_tests...
    (m, n, B, R, N_tests, filename, use_point_b, ...
     use_nonzero_omega, omega, use_state, state)
% RBK 12/16/2005 -- This takes a Gauss--Seidel step
%  for the first coordinate of the solution, for
%  the following preconditioning matrices:
% 1. The inverse midpoint preconditioner.
% 2. The optimal width LP preconditioenr
% 3. Algorithm~1 in "On Preconditioners and Splitting"
%    (Kearfott et al, 2005).
% 4. Algorithm 2 in "On Preconditioners and Splitting"
%    (Kearfott et al, 2005).
% 5. No preconditioner, but running through all equations
%    and all coordinates twice.
% 
% The computation is done for N_tests random problems,
% as defined by "random_test_matrix.m". if "use_state"
% is set to 1 on entry, then 
% the state of the uniform random generator rand is set to
% "state" before beginning to generate the N_tests test, 
% whereas the state of the random number generator is
% not pre-set if use_state is set to 0.  On return,
% "s" is set to the state of the random number generator
% at the beginning of generation of the test problems
% (so the tests can be repeated exactly, if desired).
%
% Statistics are reported as follows:
%  1. the number of times there was a successful reduction
%     in measure of the interval;
%  2. the number of splits.
%  3. the number of times the particular scheme resulted
%     in a maximum radius-reduction.
%  4. the average ratio (new radius) / (old radius).

if(use_state)
    rand('state',state);
end
    
s = rand('state');

fid = fopen(filename, 'w');
timevec = clock;
fprintf(fid, 'Output from run_random_tests on ');
fprintf(fid, '%4.0f/%2.0f/%2.0f at %2.0f:%2.0f.%2.0f.\n',...
    timevec(1:6)');
fprintf(fid,'filename:');
fprintf(fid,filename);
fprintf(fid,'\n\n');

fprintf(fid,'m = %6.0f and n = %6.0f\n', m, n);
fprintf(fid,'B = %12.4e and R = %12.4e\n', B, R);
fprintf(fid,'use_point_b = %2.0f\n', use_point_b);
fprintf(fid,'use_nonzero_omega = %2.0f,  omega = %12.4e\n',...
        use_nonzero_omega, omega);
fprintf(fid,'N_tests = %6.0f\n', N_tests);
fprintf(fid,'Beginning random state for "rand":\n');
for i=0:6;
    fprintf(fid,'\n%25.19e,%25.19e,',...
        s(5*i+1), s(5*i+2), s(5*i+3), s(5*i+4), s(5*i+5));
end
fprintf(fid,'\n\n');

n_success_reduce = zeros(5,1);
n_splits = zeros(5,1);
n_max_radius_reduction = zeros(5,1);
n_radius_reduction = zeros(5,1);
average_ratio = zeros(5,1);

reduced_radius = zeros(N_tests,5);
radius_reduction = zeros(N_tests,5);
radius_ratio = zeros(N_tests,5);

max_radius_reduction = zeros(N_tests,1);
original_radius = zeros(N_tests,1);
new_x_1 = zeros(1,n);
new_x_2 = zeros(1,n);

rho = 1000;
L = 10;
alg_time= zeros(5,1);

for i=1:N_tests
    A_center = 2*rand(m,n) - 1;
    A_radius = B*rand(m,n);
    A = midrad(A_center,A_radius);
    b_center = 2*rand(m,1) -1;
    if(use_point_b)
        b = midrad(b_center,0);
    else
        b_radius = R*rand(m,1);
        b = midrad(b_center,b_radius);
    end
    if (use_nonzero_omega)
        b_offset = omega*rand(m,1);
        b = b + b_offset;
    end
    x_radius = R*rand(n,1);
    x_center = zeros(n,1);
    x = midrad(x_center,x_radius);
    
    original_radius(i) = rad(x(1));
    for j=1:5
        tic;
        switch(j)
            case(1)
                Y = inv(mid(A));
                Y_k = Y(1,:);
                [new_x_k_s, second_new_x_k_s, there_are_2_s,...
                    num, denom] = ...
                        gauss_seidel_step(1, Y_k, A, b, x);
                [new_x_k_1, new_x_k_2, ...
                    there_are_2, is_empty, is_different]... 
                     = extended_intersect ...
                      (x(1), new_x_k_s, second_new_x_k_s, ...
                         there_are_2_s);
            case(2)
                [f, Aineq, bineq, Aeq, beq, lb, H] = ...
                    C_LP_def(0.5, x, A, b, 1, rho);
                [ypm,objmin,exitflag,output,lambda] = ...
                    linprog(f,Aineq,bineq,Aeq,beq,lb);
                Y_k = extract_y(A,ypm);
                [new_x_k_s, nsecond_new_x_k_s, there_are_2_s,...
                    num, denom] = ...
                        gauss_seidel_step(1, Y_k, A, b, x);
                [new_x_k_1, new_x_k_2, ...
                    there_are_2, is_empty, is_different]... 
                     = extended_intersect ...
                      (x(1), new_x_k_s, second_new_x_k_s, ...
                         there_are_2_s);
            case(3)
                [new_x_k_1, new_x_k_2, there_are_2,...
                  is_different, is_empty, best_precond_no]...
                  = composite_best_width_reduction(1, A, b, x, L);
            case(4)
                [new_x_k_1, new_x_k_2, there_are_2,...
                  is_different, is_empty, best_precond_no]...
                  = composite_some_width_reduction(1, A, b, x, L);
            case(5)
                [new_x_1, new_x_2, there_are_2,...
                  is_different, is_empty] ...
                  = all_coordinates_no_preconditioner(1, A, b, x);
              best_precond_no = 0;
              new_x_k_1 = new_x_1(1);
              new_x_k_2 = new_x_2(1);
            otherwise
        end
        if(~is_different)
            reduced_radius(i,j) = original_radius(i);
            radius_reduction(i,j) = 0;
        else
            n_radius_reduction(j) = n_radius_reduction(j) + 1;
            if(is_empty)
                radius_reduction(i,j) = original_radius(i);
                reduced_radius(i,j) = 0;
            else
                if(there_are_2)
                    reduced_radius(i,j) =... 
                        rad(new_x_k_1) + rad(new_x_k_2);
                    n_splits(j) = n_splits(j) + 1;
                else
                    reduced_radius(i,j) = rad(new_x_k_1);
                end
            end
            radius_reduction(i,j) = ...
                original_radius(i) - reduced_radius(i,j);
        end
        radius_ratio(i,j) = reduced_radius(i,j)/original_radius(i);
        if(radius_reduction(i,j) > max_radius_reduction(i))
            max_radius_reduction(i)=radius_reduction(i,j);
        end
        alg_time(j) = alg_time(j) + toc;
    end
end

for j=1:5
    average_ratio(j) = 0;
    for i=1:N_tests
        if((max_radius_reduction(i)-radius_reduction(i,j))...
                / original_radius(i) < 1e-2)
            n_max_radius_reduction(j) = n_max_radius_reduction(j)+1;
        end
        average_ratio(j) = average_ratio(j) + radius_ratio(i,j);
    end
    average_ratio(j) = average_ratio(j)/N_tests;
end

fprintf(fid,'no. successful reductions, no. splits, ');
fprintf(fid,'no. max. radius reduction, ave. ratio, time in sec.:\n');
for j=1:5
    switch(j)
        case(1)
            fprintf(fid,'       Inv.~mid.:&     ');
        case(2)
            fprintf(fid,'&&&&&& Opt.~width:&    ');
        case(3)
            fprintf(fid,'&&&&&& Alg.~2:&        ');
        case(4)
            fprintf(fid,'&&&&&& Alg.~3:&        ');
        case(5)
            fprintf(fid,'&&&&&& none:&          ');
    end
    fprintf(fid,'%6.0f& %6.0f& %6.0f& %3.3f& %8.2f\\\\ \n',...
        n_radius_reduction(j), n_splits(j), ...
        n_max_radius_reduction(j), average_ratio(j), alg_time(j));
end

fprintf(fid,'\n');
fprintf(fid,'Time at end of this run: ');
final_timevec = clock;
fprintf(fid, '%4.0f/%2.0f/%2.0f at %2.0f:%2.0f.%2.0f.\n',...
    final_timevec(1:6)');
fprintf(fid,'\n');

fprintf(fid,'\n');
fclose(fid);